from datetime import timedelta


def format_date(datetime_obj, fmt):
    return datetime_obj.strftime(fmt)


def day_delta(datetime_obj, number_of_days):
    return datetime_obj + timedelta(number_of_days)



from inspect import getmembers, isfunction

from jinja2 import Environment, BaseLoader

from metric_builder import template_filters


def get_custom_template_filters():
    return [f for f in getmembers(template_filters) if isfunction(f[1])]


class Metric:
    def __init__(self, query, reader):
        self.query = query
        self.reader = reader

        self.jinja_env = Environment(autoescape=True, loader=BaseLoader())
        self.__load_custom_filters()

    def fetch(self, reference_time):
        """Interpolate and execute the defined function with the passed in reader object"""
        query = self.__interpolate_query(reference_time)
        return self.reader.execute(query)

    def __load_custom_filters(self):
        """Load all custom template filters and add them to the jinja environment"""
        custom_filters = get_custom_template_filters()
        for f in custom_filters:
            self.jinja_env.filters[f[0]] = f[1]

    def __interpolate_query(self, reference_time):
        template = self.jinja_env.from_string(self.query)
        return template.render(reference_time=reference_time)



from abc import ABC, abstractmethod
from typing import Dict, Any, List

from google.cloud import bigquery


class Reader(ABC):
    @abstractmethod
    def execute(self, query) -> List[Dict[str, Any]]:
        """
        Return a list of dicts of the results
        E.g. [{'col1': 213, 'col2': 234}, {'col1': 454, 'col2': 565}]
        """
        pass


class BigQueryReader(Reader):
    def __init__(self, json_credentials_path=None):
        if json_credentials_path:
            self.client = bigquery.Client.from_service_account_json(json_credentials_path=json_credentials_path)
        else:
            self.client = bigquery.Client()

    def execute(self, query) -> List[Dict[str, Any]]:
        query_job = self.client.query(query)
        return [{item[0]:item[1] for item in row.items()} for row in query_job]





import setuptools

with open("README.md", "r") as f:
    long_description = f.read()

with open('requirements.txt') as req:
    requirements = list(filter(None, [x if 'github.com' not in x else None for x in req.read().split('\n')]))

with open('requirements.txt') as req:
    dependencies = list(filter(None, [x if 'github.com' in x else None for x in req.read().split('\n')]))

EXCLUDE_FROM_PACKAGES = ['docs', 'tests*']

setuptools.setup(
    name="metric_builder",
    version="0.0.4",
    author="Philip Perold",
    author_email="philip@spatialedge.co.za",
    description="Utility for building templated metric extraction queries that can be traversed through time.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="",
    packages=setuptools.find_packages(exclude=EXCLUDE_FROM_PACKAGES),
    install_requires=requirements,
    dependency_links=dependencies,
    classifiers=[
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 3',
        'Topic :: Utilities'
    ],
    include_package_data=True
)
